<?php
namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\FPA\Services\LoteLancEventoService;
use HCM\FPA\Services\ExportacaoExcelService;

class LoteLancEventoController extends Crud {

    protected $dataSourceGetLoteLancEvento = 'getLoteLancEvento';
    protected $dataSourceSetLoteLancEvento = 'fpaLoteLancEvento_setLoteLancEvento';


    protected $parameterBagColumns = array('NRORG', 'DTMESCOMPETENC', 'NRORGPADRAO', 'NRLOTELANCEVENTO', 'NRESTRUTLEGAL_LIST', 'NRESTRUTGEREN_LIST', 'NRESTRUTSIND_LIST');
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, LoteLancEventoService $LoteLancEventoService, ExportacaoExcelService $ExportacaoExcelService){
        $this->parameterBag    = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->LoteLancEventoService = $LoteLancEventoService;
        $this->ExportacaoExcelService = $ExportacaoExcelService;
        parent::__construct($dataSourceManager);
    }

    public function getLoteLancEvento (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetLoteLancEvento);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setLoteLancEvento (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetLoteLancEvento;
        try{
            $msg = array();
            $disponibilidade = true;
            foreach($request->getDataSet()->getRows() as $row) {
                if (isset($row['NRTIPOVINCULOM'])) {
                    $bloq = $this->LoteLancEventoService->verificaEventoBloqueado($row['NRORG'], $row['NREVENTOM'], $row['NRTIPOVINCULOM']);
                    if ($bloq !== null){
                        if ($bloq['NREVENTOMPARA'] === null) {
                            $this->msg[] = array('msg' => 'Operação bloqueada. O evento '.$row['NREVENTOM'].' não pode ser lançado para o vínculo "'.$row['NRVINCULOM'].' - '.$row['NMVINCULOM']. '"!');
                            $disponibilidade = false;
                        } else {
                            $msg[] = array('msg' => 'Operação bloqueada.  O evento '.$row['NREVENTOM'].' não pode ser lançado para o vínculo "'.$row['NRVINCULOM'].' - '.$row['NMVINCULOM'].'" e foi ajustado para o evento '.$bloq['NREVENTOMPARA'].'!');
                            $disponibilidade = true;
                            $row['NREVENTOM'] = $bloq['NREVENTOMPARA'];
                        }
                    } else {
                        $disponibilidade = true;
                    }
                }
                if ($disponibilidade === true) {
                    if($row['__is_new'] === true){
                        $row['NRLOTELANCEVENTO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_LOTELANCEVENTO', 1, 12, true);
                    } 
                    if(!isset($row['IDORIGLOTELANC'])){
                        $row['IDORIGLOTELANC'] = 'TELADELANCAMENTO';
                    }
                    if(isset($row['EXCLUIITENSZERADOS'])){ 
                        $this->LoteLancEventoService->deteleItensLoteZerados($row);
                    }
                    
                    parent::save($request, $response);
                }
            }
            if (count($msg) > 0) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet("msg", $msg));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteItensZeradosLote (DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        try {
            $dataset = $request->getDataSet()->getRows();
            $row = $dataset[0];
            
            $this->LoteLancEventoService->deteleItensLoteZerados($row);    
            
            $message = new \Zeedhi\Framework\DTO\Response\Message('Delete dos Itens do Lote finalizado com sucesso', DTO\Response\Message::TYPE_MESSAGE);
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            }
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function deleteLote (DTO\Request\DataSet $request, DTO\Response $response) {
        
        try {
            $dataset = $request->getDataSet()->getRows();
            $row = $dataset[0];
            
            $this->LoteLancEventoService->deleteLote($row);    
            
            $message = new \Zeedhi\Framework\DTO\Response\Message('Delete do Lote e Itens finalizado com sucesso', DTO\Response\Message::TYPE_MESSAGE);
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            }
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function importaArquivoParaLote (DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        try {
            $dataset = $request->getDataSet()->getRows();
            
            $row = $dataset[0];
            
            $mensagens = [];
            $erros = [];
            
            foreach ($row['DSPATH'] as $arquivo) {
                $row['DSPATH'] = [$arquivo];
                
                if($row['NMIMPORTACAO'] == 'importacao_acj'){
                    $this->LoteLancEventoService->importaArquivoAcj($row);  
                }
                
                if($row['NMIMPORTACAO'] == 'importacao_txt'){
                    $result = $this->LoteLancEventoService->importaArquivoTxt($row);
                    
                    if($result){
                        foreach ($result as $value) {
                            $erros[]["DSDESCRICAO"] = $value;
                        }
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('erros', $erros));
                    }
                }
                
                if($row['NMIMPORTACAO'] == 'importacao_txt_cpf'){             
                    $result = $this->LoteLancEventoService->importaArquivoTxtCPF($row);
                    
                    if($result){
                        foreach ($result as $value) {
                            $erros[]["DSDESCRICAO"] = $value;
                        }
                        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('erros', $erros));
                    }
                }
                
                if($row['NMIMPORTACAO'] == 'importacao_planilha'){
                    $result = $this->LoteLancEventoService->importaArquivoPlanilha($row);
                    if($result) {
                        foreach ($result as $value) {
                            $erros[]["DSDESCRICAO"] = $value;
                        }
                    } else {
                        $mensagens[] = "Arquivo '" . $row['DSPATH'][0]['name'] . "' importado com sucesso"; 
                    }
                }
                
                if($row['NMIMPORTACAO'] == 'importacao_planilha_modelo2'){
                    $result = $this->LoteLancEventoService->importaArquivoPlanilhaModelo2($row);
                    if($result) {
                        foreach ($result as $value) {
                            $erros[]["DSDESCRICAO"] = $value;
                        }
                    } else {
                        $mensagens[] = "Arquivo '" . $row['DSPATH'][0]['name'] . "' importado com sucesso"; 
                    }
                }
            }
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('erros', $erros));
            
            $message = new \Zeedhi\Framework\DTO\Response\Message($mensagens, DTO\Response\Message::TYPE_MESSAGE);
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            }
        
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function exportarTemplate(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $dataset = $request->getDataSet()->getRows();
            $row = $dataset[0];
            $file = $this->ExportacaoExcelService->gravar($row);
            if($file){
                $return = array('NMARQUIVO' => $file);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $return));
            }
            
            //$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('erros', $esse));
        //    $exportador = new ExportacaoExcelService($nrestrutlegal, $nrestrutgeren, $nrestrutsind, $nreventos, $dtmescompetencia, $nrorg, $nrtipohierarq, $nrestruthierarq);
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
   
    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}