<?php

namespace HCM\Geral\Entity;

use HCM\Util\TkEntity;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use Teknisa\Model\Gestaopessoas\Entities\GpeAltesitufunc;
use Teknisa\Model\Gestaopessoas\Entities\GpeSitufuncm;
use Setforms2\Shared\ORM\DateTime;
use HCM\FPA\Entity\FpaValorreferenciamEntity;

class GpealtesitufuncEntity extends TkEntity
{
    const SIT_FUNC_NORMAL   = 1;
    const SIT_FUNC_SAL_MATERNIDADE = 5;
    const SIT_FUNC_DEMITIDO = 13;
    const SIT_FUNC_ACIDENTE_SUP_15D = 3;
    const SIT_FUNC_ACIDENTE_15D = 16;

    private $_nrOrgTrabalho;
    private $_nrOrgPadrao;
    private $_mesCompetencia;

    public function __construct($entityManager, $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrgTrabalho = $this->hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $this->hcmParameters->getNrorgPadrao();
        $this->_mesCompetencia = $this->hcmParameters->getDtCompetencia();
        $this->alteSituFunc = Repositories::GPE_ALTESITUFUNC;
        
        parent::__construct(null, $entityManager);
    }
    
    public function retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc) {
        try {
            return $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtinisitufunc);
        } catch (\Exception $e) {
            throw $e->getMessage();
        }
    }  
    
    public function retornaSituacaoFuncionalDentroPeriodo($nrorg, $nrvinculom, $inicioPeriodo, $fimPeriodo) {
        try {
            return $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncionalDentroPeriodo($nrorg, $nrvinculom, $inicioPeriodo, $fimPeriodo);
        } catch (\Exception $e) {
            throw $e->getMessage();
        }
    }  
    
    public function retornaSituacaoFuncionalNrsituDentroPeriodo($nrorg, $nrvinculom, $inicioPeriodo, $fimPeriodo, $nrSitufuncm) {
        try {
            return $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncionalNrsituDentroPeriodo($nrorg, $nrvinculom, $inicioPeriodo, $fimPeriodo, $nrSitufuncm);
        } catch (\Exception $e) {
            throw $e->getMessage();
        }
    }
    
    public function retornaSituacaoFuncionalNrsituDentroPeriodoExclusivo($nrorg, $nrvinculom, $inicioPeriodo, $fimPeriodo, $nrSitufuncm) {
        try {
            return $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaSituacaoFuncionalNrsituDentroPeriodoExclusivo($nrorg, $nrvinculom, $inicioPeriodo, $fimPeriodo, $nrSitufuncm);
        } catch (\Exception $e) {
            throw $e->getMessage();
        }
    }

    
    public function verificaEstabilidade($nrorg, $dtcompetencia, $nrvinculom, $dtrescisao) {
        
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $ultaltesitufunc = $this->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtrescisao);
        $fimaltesitufunc = null;
        
        if($ultaltesitufunc) {
            if ($ultaltesitufunc->getDtestabsitufunc()) {
                $fimaltesitufunc = $ultaltesitufunc->getDtestabsitufunc();
            } else {
                $fimaltesitufunc = $ultaltesitufunc->getDtfimsitufunc();
            }
        }
        if ($fimaltesitufunc) {
            $nrsitufuncm = $ultaltesitufunc->getNrsitufuncm();
            
            $situfunch =  $this->entityManager->getRepository(Repositories::GPE_SITUFUNCM)->retornaUltimoHistoricoCompetencia($nrorg, $nrorgPadrao, $nrsitufuncm, $dtcompetencia);
            
            $diasestabilidade = $situfunch->getQtestabpostsitu();
            if ($diasestabilidade) {
                $fimaltesitufunc = DateUtil::adicionaIntervalo($fimaltesitufunc, $diasestabilidade);
            }
            $dtrescisao = \HCM\Util\DateUtil::getDataDeString(substr($dtrescisao,0,10),DateUtil::FORMATO_BRASILEIRO, true);
            
            if (\HCM\Util\DateUtil::comparaDatas($dtrescisao, $fimaltesitufunc) && $situfunch->getNrsitufuncm() != GpeSitufuncm::SITUACAO_FUNCIONAL_NORMAL) {
                return true;
            }
        }    
        return false;
    }
    
    public function verificaVinculoAfastado($nrorg, $nrvinculom, $dtrescisao) {
        try {
            
            $ultaltesitufunc = $this->retornaUltimaSituacaoPorDatainicio($nrorg, $nrvinculom, $dtrescisao);
            
            if ($ultaltesitufunc) {
                if($ultaltesitufunc->getNrsitufuncm() != GpeSitufuncm::SITUACAO_FUNCIONAL_NORMAL &&
                   $ultaltesitufunc->getNrsitufuncm() != GpeSitufuncm::SITUACAO_FUNCIONAL_ATESTADO) {
                    if ($ultaltesitufunc->getDtfimsitufunc()) {
                        
                        $fimaltesitufunc = $ultaltesitufunc->getDtfimsitufunc();
                        
                        $fimaltesitufunc = DateUtil::truncateString($fimaltesitufunc);
                        $fimaltesitufunc = DateUtil::getDataDeString($fimaltesitufunc);
                        if (\HCM\Util\DateUtil::comparaDatas($fimaltesitufunc, $dtrescisao)) {
                            return false;
                        }
                    }
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaAlteracaoSituacaoFuncional($nr_org, $nr_vinculo, $dtAdmissao = null) {
        $this->resetItems();
        
        $resultado = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->findBy(array('nrvinculom' => $nr_vinculo,'nrorg' => $nr_org), array('dtinisitufunc' => 'ASC'));
        if (count($resultado) === 0 && $dtAdmissao) {
            $this->incluiAltSituFuncional(
                $nr_vinculo, $nr_org, GpesitufuncmEntity::SITUACAO_FUNCIONAL_NORMAL, $dtAdmissao
            );
            $resultado = $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->findBy(array('nrvinculom' => $nr_vinculo,'nrorg' => $nr_org), array('dtinisitufunc' => 'ASC'));
        }

        foreach ($resultado as $entity_obj) {
            $this->addItem($entity_obj, $entity_obj->getNraltesitufunc());
        }
    }
    
    public function retornaAnosPerdidosAfastamento($vinculo, $dt_admissao, $dt_rescisao){
        
        if(!$this->getItems()){
            $this->retornaAlteracaoSituacaoFuncional($vinculo->getNrorg(), $vinculo->getNrvinculom(), $dt_admissao);
        }
        $items = $this->getItems();
        $AnoPerda = 0;
        $afastamentos = array(6, 3, 7);
        $valorRef = new FpaValorreferenciamEntity($this->entityManager, $this->hcmParameters);
        $diasParaPerder = $valorRef->getValorReferencia(FpaValorreferenciamEntity::DIAS_AFAST_PERDA_AVISO);
        $x = $dt_admissao;
        
        if($diasParaPerder > 0){
            for ($i = $dt_admissao; $i < $dt_rescisao; $i = DateUtil::adicionaIntervalo($i, 1, DateUtil::ANO)) {
                foreach ($items as $item) {
                    if(array_search($item->getNrsitufuncm(), $afastamentos) !== false){
                        
                        $dtIniAfastamento = $item->getDtinisitufunc();
                        $dtFimAfastamento = $item->getDtfimsitufunc();
                        //$diasAfastadoIni = DateUtil::getQtdeDiasEntre($dtIniAfastamento, $i);
                        
                        if($i > $dtIniAfastamento &&  $i < $dtFimAfastamento){
                            $AnoPerda += 1;
                        }
                    }
                }
                $x = $i;
            }
        }
        
        return $AnoPerda;
    }
    
    
    public function retornaMesesAfastamento($data_ini_per, $data_fim_per) {

        $meses_afastados = 0;
        $data_ini_afastamento = new TkDateTime();
        $data_fim_afastamento = new TkDateTime();

        foreach ($this->getItems() as $item) {
            $data_ini_afastamento->set($item->getDtinisitufunc(), TkDateTime::FORMAT_DATE);

            if ($data_ini_afastamento->isEarlier($data_ini_per) === true) {
                $data_ini_afastamento->set($data_ini_per);
            }

            if ($item->getDtfimsitufunc() !== null) {
                $data_fim_afastamento->set($item->getDtfimsitufunc(), TkDateTime::FORMAT_DATE);

                if ($data_fim_afastamento->isLater($data_fim_per) === true) {
                    $data_fim_afastamento->set($data_fim_per);
                }
            } else {
                $data_fim_afastamento->set($data_fim_per, TkDateTime::FORMAT_DATE);
            }

            $meses_afastados += $data_ini_afastamento->monthsAfter($data_fim_afastamento);
        }

        return $meses_afastados;

    }

    public function retornaDiasAfastamento(
        $situacao_funcional = null, $data_ini_per, $data_fim_per,
        $determina_prox_sf, $nrvinculom = null
    ) {
        
        $afastamento = array();
        if($data_ini_per->format('d/m/Y') == $data_fim_per->format('d/m/Y')){
            $dias_periodo = 1;
        } else {
            $dias_periodo = DateUtil::getQtdeDiasEntre($data_ini_per, $data_fim_per) + 1;
        }
        
        for ($i=0; $i < $dias_periodo; $i++) {
            $afastamento[$i] = -1;
        }
        
        $persist = false;
        $data_fim_flag = true;
        $cont_alt_sits = (count($this->getItems()));
        $cont_itera = 0;
        
        foreach ($this->getItems() as $item) {
            $cont_itera++;
            $this->_verificaProxSituacaoFuncional(
                $determina_prox_sf, $situacao_funcional, $item, $data_fim_per, $persist, $nrvinculom
            );
            
            $data_ini_aux = DateUtil::getDataDeString(substr($item->getDtinisitufunc(), 0, 10), DateUtil::FORMATO_BRASILEIRO, true);

            if ($data_ini_aux > $data_fim_per) {
                break;
            }
            
            $data_fim_aux = $this->_retornaDataFimSituacao($item->getDtfimsitufunc(), $data_fim_per, $cont_alt_sits,$cont_itera);
            
            if ((($data_ini_aux > $data_ini_per)
                ||($data_ini_aux->format('d/m/Y') == $data_ini_per->format('d/m/Y')))
                &&(($data_ini_aux < $data_fim_per)
                ||($data_ini_aux->format('d/m/Y') == $data_fim_per->format('d/m/Y')))
            ) {
                $dia_ini = DateUtil::getQtdeDiasEntre($data_ini_per, $data_ini_aux);

                if($data_ini_aux->format('d/m/Y') == $data_fim_aux->format('d/m/Y')){
                    $qtd_dias = 1;
                } else {
                    $qtd_dias = DateUtil::getQtdeDiasEntre($data_ini_aux, $data_fim_aux) + 1;
                }
                if ($dia_ini < 0){
                    $dia_ini = 0;
                }

                for ($i=0; $i < $qtd_dias; $i++) {
                    $afastamento[$i+$dia_ini] = (int)$item->getNrsitufuncm();
                }
            } elseif (($data_ini_aux < $data_ini_per)
                    &&(($data_fim_aux < $data_fim_per)
                    || ($data_fim_aux->format('d/m/Y') == $data_fim_per->format('d/m/Y'))
                    || ($data_fim_aux->format('d/m/Y') == $data_ini_per->format('d/m/Y')))
            ) {
                if (($data_fim_aux > $data_ini_per)
                    ||($data_fim_aux->format('d/m/Y') == $data_ini_per->format('d/m/Y'))
                ) {
                    if($data_ini_per->format('d/m/Y') == $data_fim_aux->format('d/m/Y')){
                        $qtd_dias = 1;
                    } else {
                        $qtd_dias = DateUtil::getQtdeDiasEntre($data_ini_per, $data_fim_aux) + 1;
                    }                    

                    for ($i=0; $i < $qtd_dias; $i++) {
                        $afastamento[$i] = (int)$item->getNrsitufuncm();
                    }
                }
            }
        }
        if ($persist) {
            $this->persist();
        }
        return $afastamento;
    }
    
    private function _determinaProximaSituacaoFuncional(
     $item, $data_fim_per, $situacao_funcional = null, $nrvinculom = null
    ) {
        $data_inicio_afastamento = clone DateUtil::getDataDeString($item->getDtinisitufunc());
        $data_fim_afastamento = $item->getDtfimsitufunc();
        $persist = false;
        if (empty($data_fim_afastamento) === true) {
            // Projeta data fim.
            $sit_func = $situacao_funcional->retornaSituacaoFuncional(
                $this->_nrOrgPadrao, $this->_nrOrgTrabalho, $item->getNrsitufuncm()
            );
            if(is_object($sit_func)) {
                $qtd_limite_afastamento = false;
                
                $qtd_limite_afastamento = (int)$sit_func->historico->getQtdiaslimitsitu();
                
                /* @todo testar se $qtd_limite_afastamento <= 0 (empty) */
                if (empty($qtd_limite_afastamento) === true
                    ||($qtd_limite_afastamento === 0)
                ) {
                    // Se nao existe limite na ha como determinar proxima situacao.
                    return $persist;
                }
    
                $data_fim_afastamento = DateUtil::getDataDeString($data_inicio_afastamento);
                $data_fim_afastamento->add(new \DateInterval('P'.($qtd_limite_afastamento-1).'D'));
                if (DateUtil::comparaDatas($data_fim_afastamento,$data_fim_per) === true) {
                    // Encerra afastamento.
                    $item->setDtfimsitufunc($data_fim_afastamento);
                    $persist = true;
                }
    
                /* Uma proxima situacao no registro de alteracao (Gpealtesitufunc) tem prioridade sobre
                 * uma proxima situacao na definicao da situacao funcional (Gpesitufunc).
                 */
                 
                $prox_situacao_func = $item->getNrproxsitufunc();
                if ($prox_situacao_func === null) {
                    $prox_situacao_func = $sit_func->historico->getNrproxsitufunc();
                }
    
                if ($prox_situacao_func !== null) {
                    $data_ini_afastamento_aux = DateUtil::adicionaIntervalo($data_fim_afastamento, 1);
    
                    $novo_item = new $this->alteSituFunc;
                    $novo_item->setNraltesitufunc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESITUFUNC', 1, 12, true));
                    $novo_item->setNrorg($this->_nrOrgTrabalho);
                    $novo_item->setNrsitufuncm($prox_situacao_func);
                    $novo_item->setNrvinculom($nrvinculom);
                    $novo_item->setDtinisitufunc($data_ini_afastamento_aux);
                    $novo_item->setDtinclusao(DateUtil::getDataAtual());
                    $novo_item->setDtultatu(DateUtil::getDataAtual());
                    $novo_item->setNrorginclusao($this->_nrOrgTrabalho);
                    $novo_item->setNrorgultatu($this->_nrOrgTrabalho);
                    
                    $this->addItem($novo_item, $novo_item->getNraltesitufunc());
                    $persist = true;
                }
    
                return $persist;
            }
        }
    }

    public function incluiAltSituFuncional($nrvinculom, $nrorg, $nrsitufuncm, $dtinisitufunc) {

        try{
            $item = new $this->alteSituFunc;

            $item->setNrvinculom($nrvinculom);
            $item->setNrorg($nrorg);
            $item->setNrsitufuncm($nrsitufuncm);
            $item->setDtinisitufunc($dtinisitufunc);
            
            $item->setDtinclusao(DateUtil::getDataAtual());
            $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $item->setDtultatu(DateUtil::getDataAtual());
            $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperultatu($this->hcmParameters->getCdoperador());
            
            $item->setNraltesitufunc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESITUFUNC', 1, 12, true));
            
            $this->entityManager->persist($item);
            $this->entityManager->flush($item);

            unset($item);

            return true;
        }catch (Exception $e){
            throw $e;
        }
    }

    public function incluiAltSituFunc($nrvinculom, $nrorg, $nrsitufuncm, $dtinisitufunc, $dtfimsitufunc = null) {

        try{
            $item = new $this->alteSituFunc;

            $item->setNrvinculom($nrvinculom);
            $item->setNrorg($nrorg);
            $item->setNrsitufuncm($nrsitufuncm);
            $item->setDtinisitufunc($dtinisitufunc);
            $item->setDtfimsitufunc($dtfimsitufunc);
            
            $item->setDtinclusao(DateUtil::getDataAtual());
            $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $item->setDtultatu(DateUtil::getDataAtual());
            $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperultatu($this->hcmParameters->getCdoperador());
            
            $item->setNraltesitufunc(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'GPE_ALTESITUFUNC', 1, 12, true));
            
            $this->entityManager->persist($item);
            $this->entityManager->flush($item);

            unset($item);

            return true;
        }catch (Exception $e){
            throw $e;
        }
    }

    public function retornaAlteSituFuncporVinculo($nrvinculo, $nrorg) {
        return $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaItem($nrvinculo, $nrorg);
    }

    public function retornaAlteSitFuncporVinculoNrsituacao($nrvinculo,$nrsituacao,$nrorg) {
        return $this->entityManager->getRepository(Repositories::GPE_ALTESITUFUNC)->retornaAlteSitFuncporVinculoNrsituacao($nrvinculo,$nrsituacao , $nrorg);
    }

    public function retornaAltSitFuncporVinculoNrsituacaoDataIni($nrvinculo,$nrsituacao,$nrorg,$dtinisitu) {
        return Gpealtesitufunc::retornaAltSitFuncporVinculoNrsituacaoDataIni($nrvinculo,$nrsituacao,$nrorg,$dtinisitu);
    }

    private function _retornaDataFimSituacao($data_fim, $data_fim_per, $cont_alt_sits,$cont_itera) {
        $data_fim_flag = true;
        if ($data_fim != null) {
            $data_fim_aux = $data_fim;
        } else {
            $data_fim_flag = false;
        }

        if (($data_fim_flag === false)
            ||($data_fim_aux > $data_fim_per)
        ) {
            $data_fim_aux = $data_fim_per;
        }

        return $data_fim_aux;
    }

    private function _verificaProxSituacaoFuncional(
        $determina_prox_sf, $situacao_funcional, $item,
     $data_fim_per, &$persist, $nrvinculom = null
    ) {
        
        if ($determina_prox_sf === true && $nrvinculom) {
            $proxSituacaoFuncional = $this->_determinaProximaSituacaoFuncional(
             $item, $data_fim_per, $situacao_funcional, $nrvinculom
            );

            if ($proxSituacaoFuncional === true) {
                $persist = true;
            }
        }
    }

    public function incluiAltSituFuncionalHistorico($nrvinculom, $nrorg, $nrsitufuncm, $registro) {

        try{

            if($registro["INICIO_AFASTAMENTO"] == "" || $registro["INICIO_AFASTAMENTO"] == null){
                $dataIni = null;
            }else{
                $dataIni = $registro["INICIO_AFASTAMENTO"]." 00:00:00";
            }
            if($registro["FIM_AFASTAMENTO"] == "" || $registro["FIM_AFASTAMENTO"] == null){
                $dataFim = null;
            }else{
                $dataFim = $registro["FIM_AFASTAMENTO"]." 00:00:00";
            }

            $tipoAfastLetra = substr($registro["TIPO_AFASTAMENTO"],0,1);
            switch($tipoAfastLetra){
                case 'E':
                    $tipoAfast = 5;
                    break;
                case 'P':
                    $tipoAfast = 6;
                    break;
                case 'T':
                    $tipoAfast = 3;
                    break;
                case 'R':
                    $tipoAfast = 1; 
                    break;
                case 'I':
                    $tipoAfast = 1;
                    break;
                default:
                    $tipoAfast = 1;

            }

            $altSituFuncional = new self();

            $altSituFuncional->setNrvinculom($nrvinculom);
            $altSituFuncional->setNrorg($nrorg);
            $altSituFuncional->setNrsitufuncm($nrsitufuncm);
            $altSituFuncional->setDtinisitufunc($dataIni);
            $altSituFuncional->setDtfimsitufunc($dataFim);
            $altSituFuncional->setNrsitufuncm($tipoAfast);
            $altSituFuncional->persist();

            unset($altSituFuncional);

            return true;
        }catch (Exception $e){
            throw $e;
        }
    }

    public function setAlteracaoSituacaoFuncionario($nrorg, $nrvinculom, $nrsitufuncm, $nrmotivoafasta, $dtinisitufunc,
                                                    $dtfimsitufunc)
    {
        $this->setNrorg($nrorg);
        $this->setNrvinculom($nrvinculom);
        $this->setDtinisitufunc($dtinisitufunc);
        $this->setDtfimsitufunc($dtfimsitufunc);
        $this->setNrsitufuncm($nrsitufuncm);
        $this->setNrmotivoafasta($nrmotivoafasta);
    }

    public function getSituacaoFuncional() {
        $situfuncm = \Gpesitufuncm::retornaSituFuncPelaChave($this->getNrsitufuncm(), $this->_nrOrgTrabalho);
        return $situfuncm;
    }

    public function procuraAltSituFunc($dataInicioPeriodo, $nrTipositufuncm){
        $itens = array();
        foreach ($this->getItems() as $item) {
            if (($dataInicioPeriodo->format('d/m/Y') == $item->getDtinisitufunc()->format('d/m/Y'))
                && $nrTipositufuncm == $item->getNrsitufuncm()) {
                $itens[] = $item;
            }
        }
        return $itens;
    }

    public function deletaAltSituFunc($alteSituFunc){
        $this->entityManager->remove($alteSituFunc);
    }
    
}

