<?php

namespace HCM\SST\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class PerfilEPIController extends Crud {

    protected $perfilEpiService;
    protected $dataSourceName = 'sst_perfilepi';
    protected $dataSourceName1 = 'sst_itemperfilepi_get';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\SST\Services\PerfilEpiService $perfilEpiService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->perfilEpiService = $perfilEpiService;
        parent::__construct($dataSourceManager);
    }

    public function getPerfilepi(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getItemPerfilepi(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName1);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setPerfilEpi(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRPERFILEPI', $row)) {
                    if ($row['NRPERFILEPI'] === '') {
                        $row['NRPERFILEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_PERFILEPI', 1, 12, true);
                    }
                } else {
                    $row['NRPERFILEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_PERFILEPI', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setItemPerfil(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "sst_itemperfilepi";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRITEMPERFILEPI', $row) && $row['NRITEMPERFILEPI'] === '') {
                    $row['NRITEMPERFILEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_ITEMPERFILEPI', 1, 12, true);
                } else {
                    $row['NRITEMPERFILEPI'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'SST_ITEMPERFILEPI', 1, 12, true);
                }
            }

            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deletePerfilEpi(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->perfilEpiService->deletePerfilEpi($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }
    
    public function deleteItemPerfil(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "sst_itemperfilepi";
        parent::delete($request, $response);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 