<?php

namespace HCM\SST\Services;

use Doctrine\ORM\EntityManager;
use Folha\Entities\SstFichamedica;
use Folha\Entities\SstExamefichamed;
use Folha\Entities\SstItemexamefichamed;
use Folha\Entities\SstDoencafichamedica;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Geral\Services\ESocialService;
use Zeedhi\Framework\DataSource\DataSet;
use HCM\FPA\Services\ExportacaoExcelService;
use Zeedhi\Framework\HTTP\Kernel;
use HCM\Util\FPA;
use Geral\Util\DateTime;
use HCM\SST\Repository\FichaMedicaRepository;
use Folha\Entities\GpeVinculom;

class FichaMedicaService {
    
    protected $entityManager;
    protected $hcmParameters;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, ESocialService $eSocialService,  ExportacaoExcelService $exportacaoExcelService, Kernel $httpKernel, FichaMedicaRepository $fichaMedicaRepository) {
        $this->entityManager          = $entityManager;
        $this->hcmParameters          = $hcmParameters;
        $this->SstFichamedica         = Repositories::SST_FICHAMEDICA;
        $this->SstExamefichamed       = Repositories::SST_EXAMEFICHAMED;
        $this->gpepessoa              = Repositories::GPE_PESSOA;
        $this->gpepessoah             = Repositories::GPE_PESSOAH;  
        $this->parcnegocio            = Repositories::PARCNEGOCIO;
        $this->eSocialService         = $eSocialService;
        $this->exportacaoExcelService = $exportacaoExcelService;
        $this->httpKernel             = $httpKernel;
        $this->fichaMedicaRepository  = $fichaMedicaRepository;
    }
    
    public function deleteFichaMedica($dataset) {
        
        $nrorg = $dataset["NRORG"];
        $nrfichamedica = $dataset["NRFICHAMEDICA"];
        $exames = SstExamefichamed::findByNrfichamedicaAndNrorg($nrfichamedica, $nrorg);
        $doenca = SstDoencafichamedica::findByNrfichamedicaAndNrorg($nrfichamedica, $nrorg);
        if (isset($exames[0])) {
            throw new \Exception(Messages::HCM05000_SST_FICHAMEDICA_EXISTS_EXAME_ERROR, 10);
        }
        if (isset($doenca[0])) {
            throw new \Exception(Messages::HCM05000_SST_FICHAMEDICA_EXISTS_DOENCA_ERROR, 10);
        }
        else {
            $this->entityManager->beginTransaction();
            $fichamedica = SstFichamedica::findOneByNrfichamedicaAndNrorg($nrfichamedica, $nrorg);
            $this->entityManager->remove($fichamedica);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    public function deleteExameFichaMed($dataset) {
        $nrorg = $dataset["NRORG"];
        $nrexamefichamed= $dataset["NREXAMEFICHAMED"];
        $itemexame = SstItemexamefichamed::findByNrexamefichamedAndNrorg($nrexamefichamed, $nrorg);
        if (isset($itemexame[0])) {
            throw new \Exception(Messages::HCM05000_SST_EXAMEFICHAMED_EXISTS_ITEMEXAME_ERROR, 10);
        }
        else {
            $this->entityManager->beginTransaction();
            $examefichamed = SstExamefichamed::findOneByNrexamefichamedAndNrorg($nrexamefichamed, $nrorg);
            $this->entityManager->remove($examefichamed);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    public function validaParametros($dataset, $response){
        $nrorg          = $this->hcmParameters->getNrorgTrabalho();
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        $nrpacnegocio   = $this->VerificaNmMedico($dataset,$dtmescompetenc, $nrorg);
        $nrestrutgeren  = $dataset['NRESTRUTGEREN_LIST'];
        $nrestrutlegal  = $dataset['NRESTRUTLEGAL_LIST'];
        $nrVinculom     = $dataset['NRVINCULOM_LIST'];
        $arrayParametro =  $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculosLegalGeren($nrorg, $nrestrutgeren, $nrestrutlegal, $dtmescompetenc, $nrVinculom);
        
        if($arrayParametro != null){
            try{    
                $retorno = array();
                $responseClone = clone $response;
                foreach($arrayParametro as $registro){                 
                    $nrVinculom = $registro["NRVINCULOM"];
                    $nrfichamedica = $this->setCadastroAutomaticoFichaMedica($dataset, $nrorg, $nrVinculom, $nrpacnegocio);
                              
                    if(array_key_exists('IDENVIAESOCIAL', $dataset) && $dataset['IDENVIAESOCIAL'] == 'S'){
                        $dataset["NRVINCULOM"] = $registro["NRVINCULOM"];
                        $dataset["NRFICHAMEDICA"] = $nrfichamedica;
                        $this->eSocialService->envioEsocialDescentralizado($dataset, $responseClone);
                     }
                     
                }

                foreach($responseClone->getDataSets() as $dataSet){
                    foreach($dataSet->getRows() as &$dataSet) { 
                        $retorno[] = $dataSet;
                    }
                    
                } 

                if(!empty($retorno)) {
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('retornoEsocial', $retorno));
                }
            
            } catch (\Exception $e) {
                throw $e;
            }
            
        }

    }
    
    public function VerificaNmMedico($dataset,$dtmescompetenc, $nrorg){
        $nrinscrconsreg = $dataset['NRINSCRCONSREG'];
        if($nrinscrconsreg != null){
            $objNrInscricao = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->retornaPessoaNrinscrconsreg($nrorg, $nrinscrconsreg, $dtmescompetenc);
            if($objNrInscricao == null){
                $nrpacnegocio =  $this->salvaParcNegocio($dataset,$nrorg);
                return $nrpacnegocio;
            }else{
                $nrpessoa = $objNrInscricao[0]["nrpessoa"];
                $objNrpacrnegocio = $this->entityManager->getRepository(Repositories::GPE_PESSOA)->findOneBy(array('nrorg' =>$nrorg, 'nrpessoa'=>$nrpessoa));
                $nrpacnegocio = $objNrpacrnegocio->getNrparcnegocio();
                return $nrpacnegocio;
            }
        }
    }
    
    public function setCadastroAutomaticoFichaMedica($dataset,$nrorg ,$nrvinculom, $nrpacnegocio){
        try {
            $this->entityManager->beginTransaction();
            
            $dtconclusao  = DateUtil::getDataDeString($dataset['DTCONCLUSAO'] , DateUtil::FORMATO_BRASILEIRO, true);
            $dtPrevista   = (isset($dataset['DTHRPREVFICHAMED'])) ? DateUtil::getDataDeString($dataset['DTHRPREVFICHAMED'] , DateUtil::FORMATO_BRASILEIRO, true) : $dtconclusao;
            
            $envioFichaMedica = new $this->SstFichamedica;
            $envioFichaMedica->setNrorg($nrorg);
            $envioFichaMedica->setNrvinculom($nrvinculom);
            $envioFichaMedica->setNrinscrconselhoatend($dataset['NRINSCRCONSREG']);
            $envioFichaMedica->setNrfichamedica(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'SST_FICHAMEDICA', 1, 12, true));
            $envioFichaMedica->setNrnaturezaexame($dataset['NRNATUREZAEXAME']);
            $envioFichaMedica->setDtconclfichamed($dtconclusao);
            $envioFichaMedica->setDthrprevfichamed($dtPrevista);
            $envioFichaMedica->setIdativo('S');
            $envioFichaMedica->setNrparcnegocio($nrpacnegocio);
            
            // set auditoria
            $envioFichaMedica->setDtinclusao(DateUtil::getDataAtual());
            $envioFichaMedica->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $envioFichaMedica->setNrorginclusao($nrorg);
            $envioFichaMedica->setDtultatu(DateUtil::getDataAtual());
            $envioFichaMedica->setCdoperultatu($this->hcmParameters->getCdoperador());
            $envioFichaMedica->setNrorgultatu($nrorg);
            $this->entityManager->persist($envioFichaMedica);
            $this->entityManager->flush($envioFichaMedica);
            $nrfichamedica = $envioFichaMedica->getNrfichamedica();
            
            $this->setCadastroAutomaticoExame($dataset,$nrorg,$nrfichamedica,$nrpacnegocio);
            
            $this->entityManager->commit();
            
            return $nrfichamedica;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
     public function setCadastroAutomaticoExame($dataset,$nrorg,$nrfichamedica,$nrpacnegocio){
        try { 
            if (isset($dataset['NRPERFILEXAME']) && $dataset['NRPERFILEXAME'] != '') {
                /*Cadastra exames da ficha a partir do perfil de exames se informado*/
                $itens = $this->entityManager->getRepository(Repositories::SST_ITEMPERFILEXAME)->findBy(array('nrorg' =>$nrorg, 'nrperfilexame'=>$dataset['NRPERFILEXAME']));
                
                foreach ($itens as $item) {
                    $exame = $this->entityManager->getRepository(Repositories::SST_EXAME)->findOneBy(array('nrorg' =>$nrorg, 'nrexame'=>$item->getNrexame()));
                    
                    $envioCadExame = new $this->SstExamefichamed;
                    $envioCadExame->setNrorg($nrorg);
                    $envioCadExame->setCdproctuss($exame->getCdproctuss());
                    $envioCadExame->setNrexame($exame->getNrexame());
                    $envioCadExame->setNmexamefichamed($exame->getNmexame());
                    $envioCadExame->setNrexamefichamed(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'SST_EXAMEFICHAMED', 1, 12, true));
                    $envioCadExame->setNrfichamedica($nrfichamedica);
                    $envioCadExame->setIdativo('S');
                    $envioCadExame->setNrparcnegocio($nrpacnegocio); 
                    $envioCadExame->setDtrealizacaoexame(DateUtil::getDataDeString($dataset['DTCONCLUSAO'] , DateUtil::FORMATO_BRASILEIRO, true));
                    
                    // set auditoria
                    $envioCadExame->setDtinclusao(DateUtil::getDataAtual());
                    $envioCadExame->setCdoperinclusao($this->hcmParameters->getCdoperador());
                    $envioCadExame->setNrorginclusao($nrorg);
                    $envioCadExame->setDtultatu(DateUtil::getDataAtual());
                    $envioCadExame->setCdoperultatu($this->hcmParameters->getCdoperador());
                    $envioCadExame->setNrorgultatu($nrorg);
                    $this->entityManager->persist($envioCadExame);
                    $this->entityManager->flush($envioCadExame);
                    
                    unset($envioCadExame);
                }
            } else {
                /*Cadastra exames da fihca a partir dos exames informandos no cadastro automatico*/
                $envioCadExame = new $this->SstExamefichamed;
                $envioCadExame->setNrorg($nrorg);
                $envioCadExame->setCdproctuss($dataset['CDPROCTUSS']);
                $envioCadExame->setNrexame($dataset['NREXAME']);
                $envioCadExame->setNmexamefichamed($dataset['NMEXAMEFICHAMED']);
                $envioCadExame->setNrexamefichamed(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'SST_EXAMEFICHAMED', 1, 12, true));
                $envioCadExame->setNrfichamedica($nrfichamedica);
                $envioCadExame->setIdativo('S');
                $envioCadExame->setNrparcnegocio($nrpacnegocio); 
                $envioCadExame->setDtrealizacaoexame((DateUtil::getDataDeString($dataset['DTCONCLUSAO'] , DateUtil::FORMATO_BRASILEIRO, true)));
                
                // set auditoria
                $envioCadExame->setDtinclusao(DateUtil::getDataAtual());
                $envioCadExame->setCdoperinclusao($this->hcmParameters->getCdoperador());
                $envioCadExame->setNrorginclusao($nrorg);
                $envioCadExame->setDtultatu(DateUtil::getDataAtual());
                $envioCadExame->setCdoperultatu($this->hcmParameters->getCdoperador());
                $envioCadExame->setNrorgultatu($nrorg);
                $this->entityManager->persist($envioCadExame);
                $this->entityManager->flush($envioCadExame);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    public function salvaParcNegocio($dataset,$nrorg) {
        try{
            $obj = new $this->parcnegocio;
            $obj->setIdinstituicao('N');
            $obj->setIdpessoafisica('S');
            $obj->setIdativo('S');
            $obj->setNrorg($nrorg);
            $obj->setNrparcnegocio(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'PARCNEGOCIO', 1, 12, true));
            $obj->setNmprincipalparc($dataset['NMPESSOA']);
            $obj->setNmsecundariparc($dataset['NMPESSOA']);
            
            // set auditoria
            $obj->setDtinclusao(DateUtil::getDataAtual());
            $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $obj->setNrorginclusao($nrorg);
            $obj->setDtultatu(DateUtil::getDataAtual());
            $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
            $obj->setNrorgultatu($nrorg);
            $this->entityManager->persist($obj);
            $this->entityManager->flush($obj);
            $nrpacnegocio = $obj->getNrparcnegocio();
            $this->inserePessoaMestre($dataset,$nrorg,$nrpacnegocio);
            return $nrpacnegocio;
            
    } catch(\Exception $e) {
        throw $e;
        }
    }
    
    public function inserePessoaMestre($dataset,$nrorg,$nrpacnegocio){
        
        try{
        
            $pessoamestre = new $this->gpepessoa;
            $pessoamestre->setNrorg($nrorg);
            $pessoamestre->setNrparcnegocio($nrpacnegocio);
            $pessoamestre->setNrpessoa(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_PESSOA', 1, 12, true));
            
             // set auditoria
            $pessoamestre->setDtinclusao(DateUtil::getDataAtual());
            $pessoamestre->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $pessoamestre->setNrorginclusao($nrorg);
            $pessoamestre->setDtultatu(DateUtil::getDataAtual());
            $pessoamestre->setCdoperultatu($this->hcmParameters->getCdoperador());
            $pessoamestre->setNrorgultatu($nrorg);
            $this->entityManager->persist($pessoamestre);
            $this->entityManager->flush($pessoamestre);
            $nrPessoa = $pessoamestre->getNrpessoa();
            $this->inserePessoaHistorico($dataset,$nrorg,$nrPessoa);
        
    } catch(\Exception $e) {
        throw $e;
        }
    }
    
    public function inserePessoaHistorico($dataset,$nrorg,$nrPessoa){

        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        
        try{
            $dataset['DTEXCONSREG'] = $dataset['DTEXCONSREG'] ?? null;
            $dataset['DTINIVIGENCIA'] =  $dataset['DTINIVIGENCIA'] ?? null;
            $dataset['NRCPFPESSOA'] = $dataset['NRCPFPESSOA'] ?? null;
            $dataset['DTFIMVIGENCIA'] = $dataset['DTFIMVIGENCIA'] ?? null;

            $pessoamestre = new $this->gpepessoah;
            $pessoamestre->setNrorg($nrorg);
            $pessoamestre->setNmPessoa($dataset['NMPESSOA']);
            $pessoamestre->setNrpessoa($nrPessoa);
            $pessoamestre->setDtmescompetenc(DateUtil::getDataDeString($dtmescompetenc , DateUtil::FORMATO_BRASILEIRO, true));
            $pessoamestre->setDtexconsreg(DateUtil::getDataDeString($dataset['DTEXCONSREG'] , DateUtil::FORMATO_BRASILEIRO, true));
            $pessoamestre->setDtinivigencia(DateUtil::getDataDeString($dataset['DTINIVIGENCIA'] , DateUtil::FORMATO_BRASILEIRO, true));
            $pessoamestre->setDtfimvigencia(DateUtil::getDataDeString($dataset['DTFIMVIGENCIA'] , DateUtil::FORMATO_BRASILEIRO, true));
            $pessoamestre->setNrinscrconsreg($dataset['NRINSCRCONSREG']);
            $pessoamestre->setNrpessoah(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'GPE_PESSOAH', 1, 12, true));
            $pessoamestre->setSgufconsreg($dataset['SGUFCONSREG']);
            $pessoamestre->setNrcpfpessoa($dataset['NRCPFPESSOA']); 
            $pessoamestre->setNrconselhoreg($dataset['NRCONSELHOREG']);
            
             // set auditoria
            $pessoamestre->setDtinclusao(DateUtil::getDataAtual());
            $pessoamestre->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $pessoamestre->setNrorginclusao($nrorg);
            $pessoamestre->setDtultatu(DateUtil::getDataAtual());
            $pessoamestre->setCdoperultatu($this->hcmParameters->getCdoperador());
            $pessoamestre->setNrorgultatu($nrorg);
            $this->entityManager->persist($pessoamestre);
            $nrPessoa = $pessoamestre->getNrpessoa();
            $this->entityManager->flush($pessoamestre);
        
    } catch(\Exception $e) {
        throw $e;
        }
    }

    public function baixarPlanilhaFichaMedica()
    {
        $cabecalho = [
            "Nr. Vinculo",
            "Matrícula",
            "Matrícula eSocial",
            "Nome Médico",
            "Conselho Regional",
            "Inscrição no Conselho",
            "UF Conselho",
            "Nome Exame",
            "Data de Conclusão",
            "Natureza do Exame",
            "Exame",
            "Código do procedimento"
        ];

        $dadosExemplo[0] = [
            "12345",
            "",
            "",
            "Nome Médico",
            "3",
            "123456",
            "MG",
            "Nome do Exame",
            "01/01/2024",
            "1",
            "1",
            "1370",
        ];
        
        return $this->exportacaoExcelService->gravarTemplate_1($this->hcmParameters->getNrorgTrabalho(), $cabecalho, 'Ficha_Medica', $dadosExemplo);
    }
    
    
    public function importarPlanilhaFichaMedica($row)
    {
        $nrorg       = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $messages    = array(); 
        $dados       = $this->getDadosFichaMedicaPlanilha($row['DSPATH'][0]);

        if(!empty($dados)) {
            foreach($dados as $lin => &$dado) {
                $res = $this->insereFichaMedicaPlanilha($nrorg, $nrorgPadrao, $dado, $lin + 2);
               
                if(!empty($res)) {
                    foreach($res as $msg)
                    $messages[] = $msg;
                }    
            }
        }

        if(empty($dados)) {
            $messages[] =  array("msg" => "Não foi possível ler os dados da planilha. Por favor, gere um novo modelo e tente novamente.", "row" => 0, "field" => "N/A");
            
        }

        return $messages;
    }

    private function getDadosFichaMedicaPlanilha($path)
    {
        $head = array(
                "NRVINCULOM",
                "CDMATRICULA",
                "CDMATRICULAESOCIAL",
                "NMPESSOA",
                "NRCONSELHOREG",
                "NRINSCRCONSREG",
                "SGUFCONSREG",
                "NMEXAMEFICHAMED",
                "DTCONCLUSAO",
                "NRNATUREZAEXAME",
                "NREXAME",
                "CDPROCTUSS"
            );
            
        $file = $this->httpKernel->factoryFromFileData($path, FPA::getTempDirectory()."/");
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        
        $dateColumns = ['DTCONCLUSAO'];
        
        $file = \PHPExcel_IOFactory::load($caminhoNovo);
        $file->setActiveSheetIndex(0);
            
        return $this->exportacaoExcelService->preparaInfo_generic($file, $head, array('NRVINCULOM','CDMATRICULA','CDMATRICULAESOCIAL'), $dateColumns);
    }
    
    private function insereFichaMedicaPlanilha($nrorg,  $nrorgPadrao, $dado, $linha)
    {   
        try {

            $mensagens          = [];
            $nrVinculom         = $dado['NRVINCULOM']          ?? null;
            $cdmatricula        = $dado['CDMATRICULA']         ?? null;
            $cdmatriculaesocial = $dado['CDMATRICULAESOCIAL']  ?? null;
            $nmPessoa           = $dado['NMPESSOA']            ?? null;
            $nrConselhoReg      = $dado['NRCONSELHOREG']       ?? null;
            $nrInscrconsreg     = $dado['NRINSCRCONSREG']      ?? null;
            $sgUFconsReg        = $dado['SGUFCONSREG']         ?? null;
            $nmExameFichaMed    = $dado['NMEXAMEFICHAMED']     ?? null;
            $dtConclusao        = $dado['DTCONCLUSAO']         ?? null;
            $nrNaturezaExame    = $dado['NRNATUREZAEXAME']     ?? null;
            $nrExame            = $dado['NREXAME']             ?? null;
            $cdProcTuss         = $dado['CDPROCTUSS']          ?? null;
            $dtmescompetenc     = $this->hcmParameters->getDtCompetencia();
            $dtconclusaoFormat  = null;
            $cdProcTussValidado = null;
            

            $validacaoVinculo = $this->validaDadosPlanilhaVinculo($nrVinculom, $cdmatricula, $cdmatriculaesocial, $linha, $nrorg, $dtmescompetenc);
            $mensagens = array_merge($mensagens, $validacaoVinculo['mensagens']);
            $vinculo = $validacaoVinculo['nrVinculom'];
            
            $mensagens = array_merge($mensagens, $this->validaDadosPlanilhaConselho($nrConselhoReg, $sgUFconsReg, $nrInscrconsreg, $linha, $nrorg, $nrorgPadrao));

             // verifica se número da inscrição esta associada a algum parceiro de negocio, caso não esteja, cria um novo parceiro utilizando os dados referente a pessoa/conselho
            if (!is_null($nrInscrconsreg) && is_numeric($nrInscrconsreg)) {
                if(!is_null($nmPessoa) && is_string($nmPessoa)){
                    $nrpacnegocio = $this->VerificaNmMedico($dado,$dtmescompetenc, $nrorg);
                } else {
                    $mensagens[] = array("msg"=>"É necessário o preenchimento do campo 'Nome Médico'", "row" => $linha, "field" => 'Nome Médico');
                }
            } else {
                $mensagens[] = array("msg"=>"É necessário o preenchimento do campo 'Inscrição no Conselho' com um valor numérico.", "row" => $linha, "field" => 'Inscrição no Conselho');
            }

            if(!is_null($dtConclusao)) {
                $dtconclusaoFormat  = DateUtil::getDataDeString($dtConclusao, DateUtil::FORMATO_BRASILEIRO, true);
            } else {
                $mensagens[] = array("msg"=>"É necessário o preenchimento do campo 'Data de Conclusão' com uma data válida.", "row" => $linha, "field" => "Data de Conclusão");
            }


            $validacaoPlanilhaExame = array_merge($mensagens, $this->validaDadosPlanilhaExame($nmExameFichaMed, $nrNaturezaExame, $nrExame, $cdProcTuss, $linha, $nrorg, $nrorgPadrao));
            $mensagens = array_merge($mensagens, $validacaoPlanilhaExame['mensagens']);
            $cdProcTussValidado = $validacaoPlanilhaExame['cdProcTuss'];

            if(empty($mensagens)) {
                try {
                    $buscaFichaMedicaExamePorData = $this->fichaMedicaRepository->buscaFichaMedicaExamePorData($nrorg, $vinculo,$dtConclusao, $cdProcTussValidado);
                    if(!empty($buscaFichaMedicaExamePorData)) {
                        $mensagens[] = array("msg" => "Foi encontrado uma ficha médica com um exame para a mesma data de conclusão e código do procedimento.", "row" => $linha, "field" => "Exame duplicado");

                        return $mensagens;
                    }

                    $this->entityManager->beginTransaction();
            
                    $nrFichaMedica = $this->setImportacaoFichaMedica($nrorg, $vinculo, $nrpacnegocio, $nrInscrconsreg, $nrNaturezaExame, $dtconclusaoFormat);
                    
                    $nrFichaMedicaExame = $this->setImportacaoFichaMedicaExame($nrorg, $cdProcTussValidado, $nmExameFichaMed, $nrExame, $nrFichaMedica, $nrpacnegocio, $dtconclusaoFormat);
                    $this->entityManager->commit();

                    return;
                } catch (\Exception $e) {
                    $this->entityManager->rollback();
                    throw $e;
                }
            }

            return $mensagens;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function validaDadosPlanilhaVinculo($nrVinculom, $cdmatricula, $cdmatriculaesocial, $linha, $nrorg, $dtmescompetenc)
    {   try {

            $mensagens = [];
            $vinculo = null;

            if(is_null($nrVinculom)) {
                if($cdmatricula){

                    $objVinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->retornaVinculoPorCdmatricula($nrorg, DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO), $cdmatricula);

                    if($objVinculom == null) {
                        $mensagens[] = array("msg"=>"O código da matrícula do vínculo não foi encontrado.", "row" => $linha, "field" => 'Matrícula');
                    }
                    else {
                        $vinculo = $objVinculom->getNrvinculom();
                    }            
                }else{
                    if($cdmatriculaesocial){
                        $objVinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaVinculoPorMatriculaESocial($nrorg, null, $cdmatriculaesocial, DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO), null);
                        if($objVinculom == null) {
                            $mensagens[] = array("msg"=>"O código da matrícula do eSocial não foi encontrado na linha", "row" => $linha, "field" => 'Matrícula eSocial');
                        }
                        else $vinculo = $objVinculom->getNrvinculom();
                    }else{
                        $mensagens[] = array("msg"=>"O código do vínculo, da matrícula do vínculo ou da matrícula do eSocial não foram encontrados. É necessário o preenchimento de pelo menos um desses campos.", "row" => $linha, "field" => "Nr. Vínculo/Matrícula/Matrícula eSocial");
                    }
                }
            } else{
                $objVinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(
                    array('nrorg' => $nrorg, 'nrvinculom' => $nrVinculom));
                if (is_null($objVinculom)) {
                    $mensagens[] = array("msg"=>"O código do vínculo, da matrícula do vínculo ou da matrícula do eSocial não foram encontrados. É necessário o preenchimento de pelo menos um desses campos.", "row" => $linha, "field" =>'Nr. Vinculo/Matrícula/Matrícula eSocial');
                } else {
                    $vinculo = $objVinculom->getNrvinculom();
                }
            }
            return [
                'mensagens' => $mensagens,
                'nrVinculom' => $vinculo
            ];

        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function validaDadosPlanilhaConselho($nrConselhoReg, $sgUFconsReg, $nrInscrconsreg, $linha, $nrorg, $nrorgPadrao) 
    {
        try {
            $mensagens = [];
            if(!is_null($nrConselhoReg) && is_numeric($nrConselhoReg)) {
                $conselhoRegionalExiste = $this->entityManager->getRepository(Repositories::GPE_CONSELHOREG)->findOneBy(
                    array('nrorg' => [$nrorg, $nrorgPadrao], 'nrconselhoreg' => $nrConselhoReg));
                
                if(!$conselhoRegionalExiste) {
                    $mensagens[] = array("msg" => "Nenhum conselho regional foi encontrado. Por favor, insira um número válido. Consulte a tela 'Conselho Regional' para obter os números corretos.", "row" => $linha,  "field" => 'Conselho Regional');
                }
            } else {
                $mensagens[] = array("msg" => "É necessário o preenchimento do campo 'Conselho Regional' com um valor numérico.", "row" => $linha, "field" => 'Conselho Regional');
            }


            if(!is_null($sgUFconsReg)) {
                $estadoExiste = $this->entityManager->getRepository(Repositories::ESTADO)->findOneBy(["cdpais" => "0055", "sgestado" => $sgUFconsReg ,"nrorg" => [$nrorg, $nrorgPadrao]]);
                if (!$estadoExiste) {
                    $mensagens[] = array("msg"=>"A UF do conselho informado não foi encontrada. Por favor, insira uma sigla válida.", "row" => $linha, "field" => 'UF do Conselho');
                }
            } else {
                $mensagens[] = array("msg"=>"É necessário o preenchimento do campo 'UF do conselho' com uma sigla.", "row" => $linha, "field" => 'UF do Conselho');
            }

            return $mensagens;
        } catch (\Exception $e) {
            throw $e;
        }
    }

   private function validaDadosPlanilhaExame($nmExameFichaMed, $nrNaturezaExame, $nrExame, $cdProcTuss, $linha, $nrorg, $nrorgPadrao)
    {   
        try {
            $mensagens = [];
            $cdProcTussFormatado = null;
            if(!is_null($nmExameFichaMed)) {
                if(strlen($nmExameFichaMed) > 100 ){
                    $mensagens[] = array("msg" => "Não é permitdo inserir mais de 100 caracteres no campo de nome.", "row" => $linha, "field" => 'Nome Exame');
                }
                else{
                    $nmExameFichaMed  = mb_strtoupper($nmExameFichaMed, "utf-8" );
                }
            } else {
                $mensagens[] = array("msg"=>"É necessário o preenchimento do campo 'Nome Exame'.", "row" => $linha, "field" => "Nome Exame");
            }

            if(!is_null($nrNaturezaExame) && is_numeric($nrNaturezaExame)) {
                $existeNaturezaExame = $this->entityManager->getRepository(Repositories::SST_NATUREZAEXAME)->findOneBy(array("nrorg" => [$nrorg, $nrorgPadrao], "nrnaturezaexame" => $nrNaturezaExame));
                if(!$existeNaturezaExame) {
                    $mensagens[] = array("msg"=>"Nenhuma natureza do exame foi encontrada. Por favor, insira um número válido. Consulte a tela 'Natureza do Exame' para obter os números corretos.", "row" => $linha, "field" => 'Natureza do Exame');
                }
            } else {
                $mensagens[] = array("msg"=>"É necessário o preenchimento do campo 'Natureza do Exame' com um valor numérico.", "row" => $linha, "field" => 'Natureza do Exame');
            }

            if(!is_null($nrExame) && is_numeric($nrExame)) {
                $existeExame = $this->entityManager->getRepository(Repositories::SST_EXAME)->findOneBy(array('nrorg' =>$nrorg, 'nrexame'=> $nrExame));

                if(!$existeExame) {
                    $mensagens[] = array("msg"=>"Nenhum exame foi encontrado. Por favor, insira um número válido. Consulte a tela 'Exame' para obter os números corretos.", "row" => $linha, "field" => 'Exame');
                }
            } else {
                $mensagens[] = array("msg"=>"É necessário o preenchimento do campo 'Exame' com um valor numérico.", "row" => $linha, "field" => 'Exame');
            }

            if(!is_null($cdProcTuss) && is_numeric($cdProcTuss)) {
                $cdProcTussFormatado = str_pad($cdProcTuss, 4, '0', STR_PAD_LEFT);

                $existeCodigoProcedimento = $this->entityManager->getRepository(Repositories::SST_CDPROCEDEXAME)->findOneBy(array('cdesocial'=> $cdProcTussFormatado));

                if(!$existeCodigoProcedimento) {
                    $mensagens[] = array("msg"=>"Nenhum código do procedimento foi encontrado. Por favor, insira um código do procedimento válido com um valor numérico até 4 dígitos.", "row" => $linha, "field" => 'Código do procedimento');
                }
            } else {
                $mensagens[] = array("msg"=>"É necessário o preenchimento do campo 'Código do procedimento' com um valor numérico até 4 dígitos.", "row" => $linha, "field" => 'Código do procedimento');
            }

            return [
                'mensagens' => $mensagens,
                'cdProcTuss' => $cdProcTussFormatado
            ];
        } catch (\Exception $e) {
            throw $e;
        }
    }


    private function setImportacaoFichaMedica($nrorg, $nrVinculom, $nrpacnegocio, $nrInscrconsreg, $nrNaturezaExame, $dtConclusao)
    {
        try {
            $envioFichaMedica = new $this->SstFichamedica;
            $envioFichaMedica->setNrorg($nrorg);
            $envioFichaMedica->setNrvinculom($nrVinculom);
            $envioFichaMedica->setNrinscrconselhoatend($nrInscrconsreg);
            $envioFichaMedica->setNrfichamedica(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'SST_FICHAMEDICA', 1, 12, true));
            $envioFichaMedica->setNrnaturezaexame($nrNaturezaExame);
            $envioFichaMedica->setDtconclfichamed($dtConclusao);
            $envioFichaMedica->setDthrprevfichamed($dtConclusao);
            $envioFichaMedica->setIdativo('S');
            $envioFichaMedica->setNrparcnegocio($nrpacnegocio);
            
            $envioFichaMedica->setDtinclusao(DateUtil::getDataAtual());
            $envioFichaMedica->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $envioFichaMedica->setNrorginclusao($nrorg);
            $envioFichaMedica->setDtultatu(DateUtil::getDataAtual());
            $envioFichaMedica->setCdoperultatu($this->hcmParameters->getCdoperador());
            $envioFichaMedica->setNrorgultatu($nrorg);
            $this->entityManager->persist($envioFichaMedica);
            $this->entityManager->flush($envioFichaMedica);
            $nrfichamedica = $envioFichaMedica->getNrfichamedica();
            
            return $nrfichamedica;
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }

    private function setImportacaoFichaMedicaExame($nrorg, $cdProcTuss, $nmExameFichaMed, $nrExame, $nrFichaMedica, $nrpacnegocio, $dtConclusao)
    {
        try {
            $envioFichaMedicaExame = new $this->SstExamefichamed;
            $envioFichaMedicaExame->setNrorg($nrorg);
            $envioFichaMedicaExame->setCdproctuss($cdProcTuss);
            $envioFichaMedicaExame->setNrexame($nrExame);
            $envioFichaMedicaExame->setNmexamefichamed($nmExameFichaMed);
            $envioFichaMedicaExame->setNrexamefichamed(\HCM\Util\NovoCodigo::geraCodigo($nrorg, 'SST_EXAMEFICHAMED', 1, 12, true));
            $envioFichaMedicaExame->setNrfichamedica($nrFichaMedica);
            $envioFichaMedicaExame->setIdativo('S');
            $envioFichaMedicaExame->setNrparcnegocio($nrpacnegocio); 
            $envioFichaMedicaExame->setDtrealizacaoexame((DateUtil::getDataDeString($dtConclusao, DateUtil::FORMATO_BRASILEIRO, true)));
            
            $envioFichaMedicaExame->setDtinclusao(DateUtil::getDataAtual());
            $envioFichaMedicaExame->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $envioFichaMedicaExame->setNrorginclusao($nrorg);
            $envioFichaMedicaExame->setDtultatu(DateUtil::getDataAtual());
            $envioFichaMedicaExame->setCdoperultatu($this->hcmParameters->getCdoperador());
            $envioFichaMedicaExame->setNrorgultatu($nrorg);
            $this->entityManager->persist($envioFichaMedicaExame);
            $this->entityManager->flush($envioFichaMedicaExame);
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
}