<?php

namespace HCM\VAB\Controllers;

use HCM\Util\Messages;
use HCM\VAB\Services\CalculoValeTransporteService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;
use Folha\Entities\FpaTipomoviment;
use HCM\Util\DateUtil;

class CalculoValeTransporteController extends Crud {

    protected $dataSourceName = 'fpa_formvales';
    protected $dataSourceName_get = 'fpa_formvales_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_CDOPERINCLUSAO');

    public function __construct (Manager $dataSourceManager,EntityManager $entityManager,  ParameterBag $parameterBag, HCMParameters $hcmParameters, CalculoValeTransporteService $calculoValeTransporteService){
        $this->entityManager = $entityManager;
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->calculoValeTransporteService = $calculoValeTransporteService;
        parent::__construct($dataSourceManager);
    }

    public function getFormvales (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);

        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        
        $rows = $dataSet->getRows();
        
        foreach($rows as $key => &$row){
            if($row['DSORDEMVALE'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIOVALE']);
                $row['DSINICIO'] = $row['NRVINCULOM_LIST'];
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = $row['LIST_NMVINCULOM'];
                $row['LIST_NMVINCULOM'] = explode(",", $row['LIST_NMVINCULOM']);
            }
            
            if($row['DSORDEMVALE'] == 'LISTA_ESTRUTURA'){
                $row['DTINIADMISSAO'] = !empty($row['DSINICIOVALE1']) ? $row['DSINICIOVALE1'] : null;
                $row['DTFIMADMISSAO'] = !empty($row['DSFIMVALE1']) ? $row['DSFIMVALE1'] : null;
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIOVALE']);
                $row['DSINICIO'] = $row['LIST_NRESTRUTLEGAL'];
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['NMESTRUTLEGAL'] = $row['LIST_NMESTRUTLEGAL'];
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['LIST_NMESTRUTLEGAL']);
                
                if($row['DSFIMVALE']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIMVALE']);
                    $row['DSFIMVALE'] = $row['LIST_NRESTRUTGEREN'];
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    $row['NMESTRUTGEREN'] = $row['LIST_NMESTRUTGEREN'];
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['LIST_NMESTRUTGEREN']);
                }
            }   
            
            if($row['DSORDEMVALE'] === "LISTA_NRESTRUTSIND"){
                $row['DTINIADMISSAO'] = !empty($row['DSINICIOVALE1']) ? $row['DSINICIOVALE1'] : null;
                $row['DTFIMADMISSAO'] = !empty($row['DSFIMVALE1']) ? $row['DSFIMVALE1'] : null;
                
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIOVALE']);
                $row['DSINICIO'] = $row['LIST_NRESTRUTLEGAL'];
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['NMESTRUTLEGAL'] = $row['LIST_NMESTRUTLEGAL'];
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['LIST_NMESTRUTLEGAL']);
                
                if($row['DSFIMVALE']) {
                    $row['LIST_NRESTRUTSIND'] = explode(',',$row['DSFIMVALE']);
                    $row['DSFIMVALE'] = $row['LIST_NRESTRUTSIND'];
                    foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                        $row['LIST_NRESTRUTSIND'][$chave] = intval($value);
                    
                    $row['NMESTRUTSIND'] = $row['LIST_NMESTRUTSIND'];
                    $row['LIST_NMESTRUTSIND'] = explode(",", $row['LIST_NMESTRUTSIND']);
                }
            }
            
            if($row['DSORDEMVALE'] == 'LISTA_TOMADOR'){
                $row['DTINIADMISSAO'] = !empty($row['DSINICIOVALE1']) ? $row['DSINICIOVALE1'] : null;
                $row['DTFIMADMISSAO'] = !empty($row['DSFIMVALE1']) ? $row['DSFIMVALE1'] : null;
                $row['LIST_NRESTRUTURAMSUP'] = explode(",", $row['DSINICIOVALE']);
                $row['DSINICIOVALE'] = $row['LIST_NRESTRUTURAMSUP'];
                foreach($row['LIST_NRESTRUTURAMSUP'] as $chave => $value)
                    $row['LIST_NRESTRUTURAMSUP'][$chave] = intval($value);
                    
                $row['NMESTRUTURAMSUP'] = $row['LIST_NMESTRUTURAMSUP'];
                $row['LIST_NMESTRUTURAMSUP'] = explode(",", $row['NMESTRUTURAMSUP']);
                
                if($row['DSFIMVALE']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIMVALE']);
                    $row['DSFIMVALE'] = $row['LIST_NRESTRUTGEREN'];
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    $row['NMESTRUTGEREN'] = $row['LIST_NMESTRUTGEREN'];
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['LIST_NMESTRUTGEREN']);
                }
            }
            
            if($row['DSORDEMVALE'] == 'LISTA_LOTE'){
                $row['NRLOTEVINC_LIST'] = explode(",", $row['DSINICIOVALE']);
                $row['DSINICIO'] = $row['NRLOTEVINC_LIST'];
                foreach($row['NRLOTEVINC_LIST'] as $chave => $value)
                    $row['NRLOTEVINC_LIST'][$chave] = intval($value);
                
                $row['NMLOTEVINC'] = $row['LIST_NMLOTEVINC'];
                $row['LIST_NMLOTEVINC'] = explode(",", $row['LIST_NMLOTEVINC']);
            }
            
            if($row['DSORDEMVALE'] == 'LISTA_HIERARQUIA'){
                $row['DTINIADMISSAO'] = !empty($row['DSINICIOVALE1']) ? $row['DSINICIOVALE1'] : null;
                $row['DTFIMADMISSAO'] = !empty($row['DSFIMVALE1']) ? $row['DSFIMVALE1'] : null;
                
                $row['NRESTRUTURAMSUP'] = explode(",", $row['DSFIMVALE']);
                $row['DSFIM'] = $row['NRESTRUTURAMSUP'];
                foreach($row['NRESTRUTURAMSUP'] as $chave => $value)
                    $row['NRESTRUTURAMSUP'][$chave] = intval($value);
                
                $row['NMESTRUTSUP'] = $row['NMESTRUTURAMSUP'];
                $row['NMESTRUTURAMSUP'] = explode(",", $row['NMESTRUTSUP']);
            }
            
        }
        
        $response->addDataSet(new DataSet($this->dataSourceName_get, $rows));
    }

    public function setFormvales (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['DSINICIOVALE1'] = null;
                $row['DSFIMVALE1'] = null;
                
                if($row['IDCALCCREDINDEV'] && $row['IDCALCCREDINDEV'] == 'S'){
                    $dtiniciocredindev = DateUtil::getDataDeString($row['DTINICIOCREDINDEV'], DateUtil::FORMATO_BRASILEIRO, true);
                    $dtfinalcredindev = DateUtil::getDataDeString($row['DTFINALCREDINDEV'], DateUtil::FORMATO_BRASILEIRO, true);
                    
                    $dtinicio = DateUtil::getDataDeString($row['DTINICIOVALE'], DateUtil::FORMATO_BRASILEIRO, true);
                    $dtfinal = DateUtil::getDataDeString($row['DTFINALVALE'], DateUtil::FORMATO_BRASILEIRO, true);
                    
                    $primeiroDiaMes = DateUtil::getPrimeiroDiaDoMes($dtinicio);
                    $ultimoDiaMes = DateUtil::getUltimoDiaDoMes($dtfinal);
                    
                    $primeiroDiaMesCredIndev = DateUtil::getPrimeiroDiaDoMes($dtiniciocredindev);
                    $ultimoDiaMesCredIndev = DateUtil::getUltimoDiaDoMes($dtiniciocredindev);
                    
                    $mesCredIndevFinal = $dtfinalcredindev->format('m');
                    $mesCredIndevInicio = $primeiroDiaMesCredIndev->format('m');
                    $anoCredIndevFinal = $dtfinalcredindev->format('Y');
                    $anoCredIndevInicio = $primeiroDiaMesCredIndev->format('Y');
                    
                    //Validação para analisar mes e ano, inicio e fim do credito indevido devem estar no mesmo periodo de mês
                    if($mesCredIndevFinal != $mesCredIndevInicio || $anoCredIndevFinal != $anoCredIndevInicio){
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Início e fim dos campos de crédito indevido devem estar dentro do mesmo mês e ano informado nos campos.', DTO\Response\Message::TYPE_MESSAGE));
                        return;
                    }
                    
                    
                    if(($primeiroDiaMesCredIndev > $ultimoDiaMes) || ($primeiroDiaMesCredIndev == $primeiroDiaMes)){
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('As datas Início e Fim do crédito indevido não podem ser maiores ou iguais que o período de cálculo do formulário do VT (modifique para uma data retroativa).', DTO\Response\Message::TYPE_MESSAGE));
                        return;
                    }
                }
                
                if (array_key_exists('CDOPERADOR', $row)) {
                    if ($row['CDOPERADOR'] === '') {
                        $row['CDOPERADOR'] = $this->hcmParameters->getCdoperador();
                    }
                } else {
                    $row['CDOPERADOR'] = $this->hcmParameters->getCdoperador();
                }
                if (!array_key_exists('NRTIPOLINHAVINC', $row)) {
                    $row['NRTIPOLINHAVINC'] = null;
                }
                if (array_key_exists('NRFORMVALE', $row)) {
                    if ($row['NRFORMVALE'] === '') {
                        $row['NRFORMVALE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_FORMVALES', 1, 12, true);
                    }
                } else {
                    $row['NRFORMVALE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_FORMVALES', 1, 12, true);
                }
                
                 // define os campos de descrição de início e fim com base no critério de seleção passado pela row
                if($row['DSORDEMVALE'] === "LISTA_VINCULO"){
                    if(is_array($row['NRVINCULOM_LIST'])){
                        $row['DSINICIOVALE'] = implode(',',$row['NRVINCULOM_LIST']);
                    }
                }
                
                if($row['DSORDEMVALE'] === "LISTA_ESTRUTURA"){
                    $row['DSINICIOVALE1'] = !empty($row['DTINIADMISSAO']) ? $row['DTINIADMISSAO'] : null;
                    $row['DSFIMVALE1'] = !empty($row['DTFIMADMISSAO']) ? $row['DTFIMADMISSAO'] : null;
                    if(is_array($row['LIST_NRESTRUTLEGAL'])){
                        $row['DSINICIOVALE'] = implode(',',$row['LIST_NRESTRUTLEGAL']);
                    }
                    if(isset($row['LIST_NRESTRUTGEREN']) && is_array($row['LIST_NRESTRUTGEREN']) && count($row['LIST_NRESTRUTGEREN']) > 0 && is_array($row['LIST_NRESTRUTGEREN'])){
                        $row['DSFIMVALE'] = implode(',',$row['LIST_NRESTRUTGEREN']);
                    }else{
                        if(is_array($row['DSFIMVALE'])){
                            $row['DSFIMVALE'] = null;
                        }
                    }
                }
                
                if($row['DSORDEMVALE'] === "LISTA_NRESTRUTSIND"){
                    $row['DSINICIOVALE1'] = !empty($row['DTINIADMISSAO']) ? $row['DTINIADMISSAO'] : null;
                    $row['DSFIMVALE1'] = !empty($row['DTFIMADMISSAO']) ? $row['DTFIMADMISSAO'] : null;

                    if(is_array($row['LIST_NRESTRUTLEGAL'])){
                        $row['DSINICIOVALE'] = implode(',',$row['LIST_NRESTRUTLEGAL']);
                    }
                    if(count($row['LIST_NRESTRUTSIND']) > 0 && is_array($row['LIST_NRESTRUTSIND'])){
                        $row['DSFIMVALE'] = implode(',',$row['LIST_NRESTRUTSIND']);
                    }else{
                        if(is_array($row['LIST_NRESTRUTSIND'])){
                            $row['DSFIMVALE'] = null;
                        }
                    }
                }
                
                if($row['DSORDEMVALE'] === "LISTA_TOMADOR"){
                    $row['DSINICIOVALE1'] = !empty($row['DTINIADMISSAO']) ? $row['DTINIADMISSAO'] : null;
                    $row['DSFIMVALE1'] = !empty($row['DTFIMADMISSAO']) ? $row['DTFIMADMISSAO'] : null;
                    if(is_array($row['LIST_NRESTRUTURAMSUP'])){
                        $row['DSINICIOVALE'] = implode(',',$row['LIST_NRESTRUTURAMSUP']);
                    }
                    if(isset($row['LIST_NRESTRUTGEREN']) && is_array($row['LIST_NRESTRUTGEREN']) && count($row['LIST_NRESTRUTGEREN']) > 0 && is_array($row['LIST_NRESTRUTGEREN'])){
                        $row['DSFIMVALE'] = implode(',',$row['LIST_NRESTRUTGEREN']);
                    }else{
                        if(is_array($row['DSFIMVALE'])){
                            $row['DSFIMVALE'] = null;
                        }
                        if(is_array($row['DSINICIOVALE'])){
                            $row['DSINICIOVALE'] = null;
                        }
                    }
                }
                
                if($row['DSORDEMVALE'] === "LISTA_LOTE"){
                    if(is_array($row['NRLOTEVINC_LIST'])){
                        $row['DSINICIOVALE'] = implode(',',$row['NRLOTEVINC_LIST']);
                    }
                }
                
                if($row['DSORDEMVALE'] === "LISTA_HIERARQUIA"){
                    $row['DSINICIOVALE1'] = !empty($row['DTINIADMISSAO']) ? $row['DTINIADMISSAO'] : null;
                    $row['DSFIMVALE1'] = !empty($row['DTFIMADMISSAO']) ? $row['DTFIMADMISSAO'] : null;
                    
                    if(isset($row['NRTIPOHIERARQORG'])){
                        $row['DSINICIOVALE'] = $row['NRTIPOHIERARQORG'];
                    }
                    if(is_array($row['NRESTRUTURAMSUP'])){
                        $row['DSFIMVALE'] = implode(',', $row['NRESTRUTURAMSUP']);
                    }
                }
                
            }
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function calcular(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $nrcalculovale = $this->hcmParameters->determinaNrCalculoFolha(FpaTipomoviment::TP_MOV_VALE_TRANSPORTE);
            $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->limpaMsgCalculoProcesso(
                $nrcalculovale, 
                $this->hcmParameters->getNrorgTrabalho(), 
                'CALCULOVT',
                $this->hcmParameters->getCdoperador() 
            );
            foreach ($request->getDataSet()->getRows() as &$row) {
                $idMemoria = $row['IDMEMORIACALC'];
                $dsordem = $row['DSORDEMVALE'];
                $dsiniciovale = $row['DSINICIOVALE'];
                $dsfimvale = $row['DSFIMVALE'];
                $dtiniciovale = $row['DTINICIOVALE'];
                $dtfinalvale = $row['DTFINALVALE'];
                $dtpagamento = $row['DTPAGAMENTOVALE'];
                $nrtipolinhavinc = $row['NRTIPOLINHAVINC'];
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                
                $this->calculoValeTransporteService->calcular($response, $idMemoria, $dsordem, $dsiniciovale, $dsfimvale, $dtiniciovale, $dtfinalvale, $dtpagamento, $nrtipolinhavinc, $dtmescompetenc, $nrorg, $row);
            }
            
            $message = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(
                [
                    'nrorg'          => $this->hcmParameters->getNrorgTrabalho(),
                    'dsorigmsg'      => 'CALCULOVT',
                    'nrcalculofolha' => $nrcalculovale, 
                    'cdoperinclusao' => $this->hcmParameters->getCdoperador(),
                ]
            );
            if( count($message) > 0 ){
                $row = array();
                foreach( $message as $mensagem ){
                    $row[] = [
                              "NRMSGCALC"   => $mensagem->getNrmsgcalc(), 
                              "DSDESCRICAO" => $mensagem->getDescricao(), 
                              "IDTIPOMSG"   => $mensagem->getIdtipomsg() === 'A' ? 'Alerta' : 'Erro',
                              "DTINCLUSAO"  => $mensagem->getDtinclusao()->format('d/m/Y'),
                            ];
                }
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('calculoValeTransporte', $row));                    
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function verificaQuantidadeRetrocessoVale(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dsordem = $row['DSORDEMVALE'];
                $dsiniciovale = $row['DSINICIOVALE'];
                $dsfimvale = $row['DSFIMVALE'];
                $nrtipolinhavinc = $row['NRTIPOLINHAVINC'];
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                
                $qtdRetrocesso = $this->calculoValeTransporteService->verificaQtdRetrocessoVale($dsordem, $dsiniciovale, $dsfimvale, $nrtipolinhavinc, $dtmescompetenc, $nrorg, $row);
                if(!is_string($qtdRetrocesso)){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('O retrocesso será realizado para  '.$qtdRetrocesso.' lançamentos. Deseja realmente continuar?', 'C'));
                } else {
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($qtdRetrocesso));
                }
                
                return;
            }
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retroceder(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dsordem = $row['DSORDEMVALE'];
                $dsiniciovale = $row['DSINICIOVALE'];
                $dsfimvale = $row['DSFIMVALE'];
                $nrtipolinhavinc = $row['NRTIPOLINHAVINC'];
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                
                // Por ser retrocesso deve-se gravar o log da operação
                $this->calculoValeTransporteService->persistirLogRetrocesso($row);
                
                $this->calculoValeTransporteService->retroceder($dsordem, $dsiniciovale, $dsfimvale, $nrtipolinhavinc, $dtmescompetenc, $nrorg, $row);
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}