<?php

namespace HCM\VAB\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class TipoBeneficioController extends Crud {
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_NRTIPOBENEFICIO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getTipoBeneficio (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = "vab_tpbeneficio_get";
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setTipoBeneficio (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = 'vab_tpbeneficio_set';
    
        foreach($request->getDataSet()->getRows() as &$row) {
            if(array_key_exists('NRTPBENEFICIO', $row)){
                if($row['NRTPBENEFICIO'] === '' ){
                    $row['NRTPBENEFICIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'TPBENEFICIO', 1, 12, true);
                }
            } else {
                $row['NRTPBENEFICIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'TPBENEFICIO', 1, 12, true);
            }
        }
        
        $row['NRINTEGTRANSPM'] = !empty($row['NRINTEGTRANSPM']) ? $row['NRINTEGTRANSPM'] : null;
            
        parent::save($request, $response);
        return true;
    }
    
    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = 'vab_tpbeneficio_get';
        parent::delete($request, $response);
    }
    
    public function getTipoBeneficioGerencial (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = "vab_tpbeneficio_gerencial_get";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setTipoBeneficioGerencial (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = 'vab_tpbeneficio_gerencial_set';
        
        //Todo ajustar para criar o novo codigo da tabela.
        foreach($request->getDataSet()->getRows() as &$row) {
            if(array_key_exists('NRTPBENEFICIOGERENCIAL', $row)){
                if($row['NRTPBENEFICIOGERENCIAL'] === '' ){
                    $row['NRTPBENEFICIOGERENCIAL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'TPBENEFICIOGERENCIAL', 1, 12, true);
                }
            } else {
                $row['NRTPBENEFICIOGERENCIAL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'TPBENEFICIOGERENCIAL', 1, 12, true);
            }
        }

        parent::save($request, $response);
        return true;
    }
    
    public function deleteGerencial(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = 'vab_tpbeneficio_gerencial_get';
        parent::delete($request, $response);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}